@@ -6,7 +6,6 @@ import sys
 import ruamel.yaml
 import pdb
 import traceback
-import pdb
 import logging
 import shutil
 from caracal.dispatch_crew import config_parser
@@ -163,24 +162,28 @@ def log_logo():
 def execute_pipeline(options, config, block):
     # setup piping infractructure to send messages to the parent
     workers_directory = os.path.join(caracal.pckgdir, "workers")
+    backend = config['general']['backend']
+    if options.container_tech and options.container_tech != 'default':
+        backend = options.container_tech
+
     def __run(debug=False):
         """ Executes pipeline """
 #        with stream_director(log) as director:  # stdout and stderr needs to go to the log as well -- nah
 
         try:
-            # Obtain some divine knowledge
-            cdb = mkct.calibrator_database()
-
             pipeline = worker_administrator(config,
                            workers_directory,
                            add_all_first=False, prefix=options.general_prefix,
                            configFileName=options.config, singularity_image_dir=options.singularity_image_dir,
-                           container_tech=options.container_tech, start_worker=options.start_worker,
+                           container_tech=backend, start_worker=options.start_worker,
                            end_worker=options.end_worker, generate_reports=not options.no_reports)
 
             if options.report:
                 pipeline.regenerate_reports()
             else:
+                ## OMS: I don't think this is necessary, as it is not used here directly, and loaded on-demand
+                # # Obtain some divine knowledge
+                # cdb = mkct.calibrator_database()
                 pipeline.run_workers()
         except SystemExit as e:
             # if e.code != 0:
@@ -227,6 +230,8 @@ def main(argv):
             parser.error("unknown worker '{}'".format(options.worker_help))
         return
 
+    caracal.log.info(f"Invoked as {' '.join(sys.argv)}")
+
     # User requests default config => dump and exit
     if options.get_default:
         sample_config = SAMPLE_CONFIGS.get(options.get_default_template)
@@ -253,7 +258,7 @@ def main(argv):
             sys.exit(1)  # indicate failure
         log.info("Initializing {1} from config template '{0}' (schema version {2})".format(options.get_default_template,
                                                                                            options.get_default, version))
-        shutil.copy2(sample_config_path, options.get_default)
+        shutil.copyfile(sample_config_path, options.get_default)
         return
 
     if options.print_calibrator_standard:
