@@ -26,16 +26,17 @@ def worker(pipeline, recipe, config):
     dist_ncpu = config['calibrate_dd']['dist_ncpu']
     label = config['label_in']
     USEPB = config['use_pb']
-    pipeline.set_cal_msnames(label)
-    mslist = pipeline.cal_msnames
-    hires_mslist = pipeline.hires_msnames
+    #pipeline.set_cal_msnames(label)
+    #mslist = pipeline.cal_msnames
+    #hires_mslist = pipeline.hires_msnames
     prefix = pipeline.prefix
     INPUT=pipeline.input
     DD_DIR = "3GC"
-    OUTPUT=pipeline.output+"/"+DD_DIR
+    #OUTPUT=pipeline.output+"/"+DD_DIR
+    OUTPUT = os.path.join(pipeline.output, DD_DIR)
     DDF_LSM = "DDF_lsm.lsm.html"
-    all_targets, all_msfile, ms_dict = utils.target_to_msfiles(
-        pipeline.target, pipeline.msnames, label)
+    shared_mem = str(config['shared_mem'])+'gb'
+    all_targets, all_msfile, ms_dict = pipeline.get_target_mss(label)
     print("All_targets", all_targets)
     print("All_msfiles", all_msfile)
     #print("ms_dict",ms_dict)
@@ -54,7 +55,7 @@ def worker(pipeline, recipe, config):
     print(de_targets)
 
     dd_image_opts = {
-        "Data-MS"        : mslist,
+        "Data-MS"        : all_msfile,
         "Data-ColName"   : config['image_dd']['data_colname'],
         "Data-ChunkHours"       : config['image_dd']['data_chunkhours'],
         "Output-Mode"           : config['image_dd']['output_mode'],
@@ -94,7 +95,9 @@ def worker(pipeline, recipe, config):
         recipe.add("cab/eidos", "make-pb", eidos_opts,
         input=INPUT,
         output=OUTPUT,
-        label="make-pb:: Generate primary beams from Eidos",)
+        label="make-pb:: Generate primary beams from Eidos",
+        shared_memory=shared_mem)
+
 
     def dd_precal_image(field,ms_list):
         dd_image_opts_precal = copy.deepcopy(dd_image_opts)
@@ -108,8 +111,8 @@ def worker(pipeline, recipe, config):
             recipe.add("cab/ddfacet", "ddf_image-for_mask-{0:s}".format(field), dd_image_opts_precal,
                  input=INPUT,
                  output=OUTPUT,
-                 shared_memory="500gb",
-                 label="ddf_image-for_mask-{0:s}:: DDFacet image for masking".format(field))
+                 label="ddf_image-for_mask-{0:s}:: DDFacet image for masking".format(field),
+                 shared_memory=shared_mem)
 
 
             imname = '{0:s}{1:s}.app.restored.fits'.format(image_prefix_precal,"-DD-masking")
@@ -123,7 +126,7 @@ def worker(pipeline, recipe, config):
                  'overlap': config['image_dd']['mask_overlap'],
                  'no-negative': True,
                  'tolerance': config['image_dd']['mask_tol'],
-                 }, input=INPUT, output = OUTPUT, label='mask_ddf-precal-{0:s}:: Make a mask for the initial ddf image'.format(field))
+                 }, input=INPUT, output = OUTPUT, label='mask_ddf-precal-{0:s}:: Make a mask for the initial ddf image'.format(field),shared_memory=shared_mem)
             recipe.run()
             recipe.jobs = []
         dd_imagename = {"Output-Name": image_prefix_precal+"-DD-precal"}
@@ -134,18 +137,19 @@ def worker(pipeline, recipe, config):
         recipe.add("cab/ddfacet", "ddf_image-{0:s}".format(field), dd_image_opts_precal,
                     input=INPUT,
                     output=OUTPUT,
-                    shared_memory="500gb",
-                    label="ddf_image-{0:s}:: DDFacet initial image for DD calibration".format(field))
+                    label="ddf_image-{0:s}:: DDFacet initial image for DD calibration".format(field),shared_memory=shared_mem)
         recipe.run()
         recipe.jobs = []
+
     def dd_postcal_image(field,ms_list):
         dd_image_opts_postcal = copy.deepcopy(dd_image_opts)
         outdir = field+"_ddcal/"
         image_prefix_postcal = "/"+outdir+"/"+prefix+"_"+field
         dd_ms_list = {"Data-MS" : ms_list}
-        print("Imaging",ms_list)
-        dd_imagecol = {"Data-ColName": "SUBDD_DATA"}
         dd_image_opts_postcal.update(dd_ms_list)
+        print("Imaging",ms_list)
+        postcal_datacol = config['image_dd']['data_colname_postcal']
+        dd_imagecol = {"Data-ColName": postcal_datacol}
         dd_image_opts_postcal.update(dd_imagecol)
         if (use_mask):
             dd_imagename = {"Output-Name": image_prefix_postcal+"-DD-masking"}
@@ -154,33 +158,30 @@ def worker(pipeline, recipe, config):
                  input=INPUT,
                  output=OUTPUT,
                  label="ddf_image-postcal-{0:s}:: Primary beam corrected image".format(field),
-                 shared_memory="500gb")
+                 shared_memory=shared_mem)
             imname = '{0:s}{1:s}.app.restored.fits'.format(image_prefix_postcal,"-DD-masking")
             output_folder = "/"+outdir
             recipe.add("cab/cleanmask", "mask_ddf-postcal-{0:s}".format(field),{
-                 #'image' : '{0:s}:output'.format(imagename),
                  'image' : '{0:s}:output'.format(imname),
-                 'output' : '{0:s}mask_ddf_precal_{1:s}.fits:output'.format(output_folder,field),
+                 'output' : '{0:s}mask_ddf_postcal_{1:s}.fits:output'.format(output_folder,field),
                  'sigma' : config['image_dd']['mask_sigma'],
                  'boxes' : config['image_dd']['mask_boxes'],
                  'iters' : config['image_dd']['mask_niter'],
                  'overlap': config['image_dd']['mask_overlap'],
                  'no-negative': True,
                  'tolerance': config['image_dd']['mask_tol'],
-                 }, input=INPUT, output = OUTPUT, label='mask_ddf-postcal-{0:s}:: Make a mask for the initial ddf image'.format(field))
+                 }, input=INPUT, output = OUTPUT, label='mask_ddf-postcal-{0:s}:: Make a mask for the initial ddf image'.format(field),shared_memory=shared_mem)
             recipe.run()
             recipe.jobs = []
 
-        if use_mask:
-            dd_maskopt = {"Mask-External" : "mask_ddf_postcal_{0:s}.fits:output".format(field)}
-            dd_image_opts_postcal.update(dd_maskopt)
-
-
-        dd_beamopts = {"Beam-Model": "FITS", "Beam-FITSFile":prefix+"'_$(corr)_$(reim).fits':output", "Beam-FITSLAxis": 'px', "Beam-FITSMAxis":"py", "Output-Images": 'dmcriDMCRIPMRIikz'}
-        dd_image_opts_postcal.update(dd_ms_list)
         dd_imagename = {"Output-Name": image_prefix_postcal+"-DD-postcal"}
         dd_image_opts_postcal.update(dd_imagename)
-        dd_image_opts_postcal.update(dd_imagecol)
+
+        if use_mask:
+            dd_maskopt = {"Mask-External" : "{0:s}mask_ddf_postcal_{1:s}.fits:output".format(output_folder,field)}
+            dd_image_opts_postcal.update(dd_maskopt)
+
+        dd_beamopts = {"Beam-Model": "FITS", "Beam-FITSFile":prefix+"'_$(corr)_$(reim).fits':output", "Beam-FITSLAxis": 'px', "Beam-FITSMAxis":"py", "Output-Images": 'dmcriDMCRIPMRIikz'}
         if USEPB:
             dd_image_opts_postcal.update(dd_beamopts)
 
@@ -188,7 +189,7 @@ def worker(pipeline, recipe, config):
                     input=INPUT,
                     output=OUTPUT,
                     label="ddf_image-postcal-{0:s}:: Primary beam corrected image".format(field),
-                    shared_memory="500gb")
+                    shared_memory=shared_mem)
 
 #    def sfind_intrinsic():
 #        DDF_INT_IMAGE = prefix+"-DD-precal.int.restored.fits:output"
@@ -239,7 +240,7 @@ def worker(pipeline, recipe, config):
            }
 
            recipe.add('cab/catdagger', 'tag_sources-auto_mode', catdagger_opts,input=INPUT,
-              output=OUTPUT+"/"+outdir,label='tag_sources-auto_mode::Tag dE sources with CatDagger')
+              output=OUTPUT+"/"+outdir,label='tag_sources-auto_mode::Tag dE sources with CatDagger',shared_memory=shared_mem)
 
         if de_sources_mode == 'manual':
            img = prefix+"_"+field+"-DD-precal.app.restored.fits"
@@ -285,7 +286,7 @@ def worker(pipeline, recipe, config):
         print("test_path",test_path)
         print("output_cubical",output_cubical)
         for ms in mslist:
-           mspref = ms.split('.ms')[0].replace('-','_')
+           mspref =os.path.splitext(ms)[0].replace('-','_')
            step = 'dd_calibrate-{0:s}-{1:s}'.format(mspref,field)
            recipe.add('cab/cubical_ddf', step, {
               "data-ms"           : ms,
@@ -306,7 +307,7 @@ def worker(pipeline, recipe, config):
               "dd-max-post-error"  : config[key]['dd_dd_max_post_error'],
               "g-time-int"        : config[key]['dd_g_timeslots_int'],
               "g-freq-int"        : config[key]['dd_g_chan_int'],
-              "dist-ncpu"         :  0,
+              "dist-ncpu"         : config[key]['dist_ncpu'],
               "dist-nworker"      : config[key]['dist_nworker'],
               "dist-max-chunks"   : config[key]['dist_nworker'],
               "dist-max-chunks"   : config[key]['dist_nworker'],
@@ -348,13 +349,13 @@ def worker(pipeline, recipe, config):
                input=INPUT,
                #output=OUTPUT+"/"+outdir,
                output=output_cubical,
-               shared_memory="400gb",
+               shared_memory=shared_mem,
                label='dd_calibrate-{0:s}-{1:s}:: Carry out DD calibration'.format(mspref,field))
 
     def cp_data_column(field,mslist):
         outdir = field+"_ddcal"
         for ms in mslist:
-           mspref = ms.split('.ms')[0].replace('-','_')
+           mspref =os.path.splitext(ms)[0].replace('-','_')
            step = 'cp_datacol-{0:s}-{1:s}'.format(mspref,field)
            recipe.add('cab/msutils', step, {
                "command" : 'copycol',
@@ -364,14 +365,14 @@ def worker(pipeline, recipe, config):
                               },
                input=INPUT,
                output=OUTPUT+"/"+outdir,
-               label='cp_datacol-{0:s}-{1:s}:: Copy SUBDD_DATA to CORRECTED_DATA'.format(mspref,field))
+               label='cp_datacol-{0:s}-{1:s}:: Copy SUBDD_DATA to CORRECTED_DATA'.format(mspref,field),shared_memory=shared_mem)
 
     def img_wsclean(mslist,field):
         key='image_wsclean'
         outdir = field+"_ddcal"
         imweight = config[key]['img_ws_weight']
         pref = "DD_wsclean"
-        mspref = mslist[0].split('.ms')[0].replace('-','_')
+        mspref = os.path.splitext(mslist[0])[0].replace('-','_')
         step = 'img_wsclean-{0:s}-{1:s}'.format(mspref,field)
         recipe.add('cab/wsclean', step, {
             "msname": mslist,
@@ -380,7 +381,7 @@ def worker(pipeline, recipe, config):
             "nmiter": sdm.dismissable(config[key]['img_ws_nmiter']),
             "npix": config[key]['img_ws_npix'],
             "padding": config[key]['img_ws_padding'],
-            "scale": config[key]['img_ws_cell', cell],
+            "scale": config[key]['img_ws_cell'],
             "prefix": '{0:s}_{1:s}'.format(pref, field),
             "niter": config[key]['img_ws_niter'],
             "mgain": config[key]['img_ws_mgain'],
@@ -398,7 +399,8 @@ def worker(pipeline, recipe, config):
         },
         input=INPUT,
         output=OUTPUT+"/"+outdir,
-        label='img_wsclean-{0:s}-{1:s}:: Image DD-calibrated data with WSClean'.format(mspref,field))
+        version='2.6' if config[key]['img_ws_multi_scale'] else None,
+        label='img_wsclean-{0:s}-{1:s}:: Image DD-calibrated data with WSClean'.format(mspref,field),shared_memory=shared_mem)
 
     def run_crystalball(mslist,field):
         key='transfer_model_dd'
@@ -406,15 +408,13 @@ def worker(pipeline, recipe, config):
         pref = "DD_wsclean"
         crystalball_model = '{0:s}_{1:s}-sources.txt'.format(pref, field)
         for ms in mslist:
-           mspref = ms.split('.ms')[0].replace('-','_')
+           mspref =os.path.splitext(ms)[0].replace('-','_')
            step = 'crystalball-{0:s}-{1:s}'.format(mspref,field)
            recipe.add('cab/crystalball', step, {
                "ms": ms,
                "sky-model": crystalball_model+':output',
-               "spectra": config[key]['dd_spectra'],
                "row-chunks": config[key]['dd_row_chunks'],
                "model-chunks": config[key]['dd_model_chunks'],
-               "exp-sign-convention": config[key]['dd_exp_sign_convention'],
                "within": sdm.dismissable(config[key]['dd_within'] or None),
                "points-only": config[key]['dd_points_only'],
                "num-sources": sdm.dismissable(config[key]['dd_num_sources']),
@@ -422,15 +422,13 @@ def worker(pipeline, recipe, config):
                "memory-fraction": config[key]['dd_mem_frac'],
              },
                input=INPUT,
-               output=OUTPUT+"/"+outdir,
+               output=OUTPUT+"/"+outdir,shared_memory=shared_mem,
                label='crystalball-{0:s}-{1:s}:: Run Crystalball'.format(mspref,field))
 
     for target in de_targets:
        mslist = ms_dict[target]
        field = utils.filter_name(target)
        print("Processing field",field,"for de calibration:")
-       print(mslist)
-#       print(field)
        if USEPB:
           make_primary_beam()
        if pipeline.enable_task(config,'image_dd'):
@@ -444,7 +442,7 @@ def worker(pipeline, recipe, config):
        if pipeline.enable_task(config, 'copy_data'):
           cp_data_column(field,mslist)
        if pipeline.enable_task(config, 'image_wsclean'):
-          img_wsclean(mslist,field)
+        img_wsclean(mslist,field)
        if pipeline.enable_task(config,'transfer_model_dd'):
           run_crystalball(mslist,field)
 
