@@ -9,6 +9,7 @@ import re
 import caracal
 import sys
 import glob
+import fnmatch
 import numpy as np
 
 NAME = 'Flag'
@@ -19,54 +20,20 @@ def worker(pipeline, recipe, config):
     wname = pipeline.CURRENT_WORKER
     flags_before_worker = '{0:s}_{1:s}_before'.format(pipeline.prefix, wname)
     flags_after_worker = '{0:s}_{1:s}_after'.format(pipeline.prefix, wname)
-    if pipeline.virtconcat:
-        msnames = [pipeline.vmsname]
-        prefixes = [pipeline.prefix]
-        nobs = 1
-    else:
-        msnames = pipeline.msnames
-        prefixes = pipeline.prefixes
-        nobs = pipeline.nobs
 
+    nobs = pipeline.nobs
     msiter=0
     for i in range(nobs):
-        # loop over all input .MS files
-        # the additional 'for loop' below loops over all single target .MS files
-        #   produced by the pipeline (see "if config['field']" below)
-
-        prefix = pipeline.prefixes[i]
-
-        '''GET LIST OF INPUT MS'''
-        mslist = []
-        msn = pipeline.msnames[i][:-3]
-
-        if label == '':
-          mslist.append(pipeline.msnames[i])
-
-        elif config['field'] == 'target':
-           target_ls = pipeline.target[i]
-           for target in target_ls:
-                field = utils.filter_name(target)
-                mslist.append('{0:s}-{1:s}_{2:s}.ms'.format(msn, field, label))
-
-        elif config['field'] == 'calibrators':
-            mslist.append('{0:s}_{1:s}.ms'.format(msn, label))
-
-        for m in mslist:
-            if not os.path.exists(os.path.join(pipeline.msdir, m)):
-                raise IOError(
-                    "MS file {0:s} does not exist. Please check that is where it should be.".format(m))
+        prefix_msbase = pipeline.prefix_msbases[i]
+        mslist  = pipeline.get_mslist(i, label, target=(config['field'] == "target"))
+        target_ls = pipeline.target[i] if config['field'] == "target" else []
 
         for j, msname in enumerate(mslist):
-            msinfo = '{0:s}/{1:s}-obsinfo.json'.format(
-                pipeline.obsinfo, msname[:-3])
+            msdict = pipeline.get_msinfo(msname)
+            prefix = os.path.splitext(msname)[0]
 
-            if not os.path.exists(msinfo):
-                raise IOError(
-                    "MS info file {0:s} does not exist. Please check that is where it should be.".format(msinfo))
-
-            with open(msinfo, 'r') as stdr:
-                msdict = yaml.load(stdr)
+            if not os.path.exists(os.path.join(pipeline.msdir, msname)):
+                raise IOError("MS file {0:s} does not exist. Please check that is where it should be.".format(msname))
 
             # Write/rewind flag versions
             available_flagversions = manflags.get_flags(pipeline, msname)
@@ -80,7 +47,7 @@ def worker(pipeline, recipe, config):
                         version = flags_before_worker
                     stop_if_missing = True
                 if version in available_flagversions:
-                    if available_flagversions.index(flags_before_worker) < available_flagversions.index(version) and not config['overwrite_flagvers']:
+                    if flags_before_worker in available_flagversions and available_flagversions.index(flags_before_worker) < available_flagversions.index(version) and not config['overwrite_flagvers']:
                         manflags.conflict('rewind_too_little', pipeline, wname, msname, config, flags_before_worker, flags_after_worker)
                     substep = 'version-{0:s}-ms{1:d}'.format(version, msiter)
                     manflags.restore_cflags(pipeline, recipe, version, msname, cab_name=substep)
@@ -127,7 +94,7 @@ def worker(pipeline, recipe, config):
                     if tfld:
                         fields += tfld
                 fields = list(set(fields))
-            field_ids = utils.get_field_id(msinfo, fields)
+            field_ids = utils.get_field_id(msdict, fields)
             fields = ",".join(fields)
 
             if pipeline.enable_task(config, 'unflag'):
@@ -154,8 +121,8 @@ def worker(pipeline, recipe, config):
                 recipe.add("cab/politsiyakat_autocorr_amp", step,
                            {
                                "msname": msname,
-                               "field": fields,
-                               "cal_field": fields,
+                               "field": ",".join([str(id) for id in field_ids]),
+                               "cal_field": ",".join([str(id) for id in field_ids]),
                                "scan_to_scan_threshold": config["flag_autopowerspec"]["scan_thr"],
                                "antenna_to_group_threshold": config["flag_autopowerspec"]["ant_group_thr"],
                                "dpi": 300,
@@ -244,16 +211,25 @@ def worker(pipeline, recipe, config):
             if pipeline.enable_task(config, 'flag_spw'):
                 step = '{0:s}-spw-ms{1:d}'.format(wname, msiter)
                 flagspwselection = config['flag_spw']['chans']
+                firsts = [min(ff) for ff in msdict['SPW']['CHAN_FREQ']]
+                lasts = [max(ff) for ff in msdict['SPW']['CHAN_FREQ']]
+                nrs = msdict['SPW']['NUM_CHAN']
+                nspws = len(nrs)
                 found_valid_data = 0
                 if config['flag_spw']['ensure_valid']:
                     scalefactor, scalefactor_dict = 1, {
-                        'GHz': 1e+9, 'MHz': 1e+6, 'kHz': 1e+3}
+                        'GHz': 1e+9, 'MHz': 1e+6, 'kHz': 1e+3 }
                     for ff in flagspwselection.split(','):
+                        found_units = False
                         for dd in scalefactor_dict:
                             if dd.lower() in ff.lower():
                                 ff, scalefactor = ff.lower().replace(
                                     dd.lower(), ''), scalefactor_dict[dd]
-                        ff = ff.lower().replace('hz', '').split(':')
+                                found_units = True
+                        if 'hz' in ff.lower():
+                          ff = ff.lower().replace('hz', '')
+                          found_units = True
+                        ff = ff.split(':')
                         if len(ff) > 1:
                             spws = ff[0]
                         else:
@@ -261,7 +237,7 @@ def worker(pipeline, recipe, config):
                         edges = [
                             ii*scalefactor for ii in map(float, ff[-1].split('~'))]
                         if '*' in spws:
-                            spws = list(range(len(pipeline.firstchanfreq[i])))
+                            spws = list(range(nspws))
                         elif '~' in spws:
                             spws = list(
                                 range(int(spws.split('~')[0]), int(spws.split('~')[1])+1))
@@ -269,7 +245,9 @@ def worker(pipeline, recipe, config):
                             spws = [int(spws), ]
                         edges = [edges for uu in range(len(spws))]
                         for ss in spws:
-                            if ss < len(pipeline.lastchanfreq[i]) and min(edges[ss][1], pipeline.lastchanfreq[i][ss])-max(edges[ss][0], pipeline.firstchanfreq[i][ss]) > 0:
+                            if found_units and ss < nspws and min(edges[ss][1], lasts[ss]) - max(edges[ss][0], firsts[ss]) > 0:
+                                found_valid_data = 1
+                            elif not found_units and ss < nspws and edges[ss][0]>=0 and edges[ss][1] < nrs[ss]:
                                 found_valid_data = 1
                     if not found_valid_data:
                         caracal.log.warn(
@@ -380,13 +358,39 @@ def worker(pipeline, recipe, config):
                                "msname": msname,
                                "mask": config['flag_mask']['mask'],
                                "accumulation_mode": 'or',
-                               "uvrange": sdm.dismissable(config['flag_mask']['uvrange']),
+                               "uvrange": sdm.dismissable(config['flag_mask']['uvrange'] or None),
                                "memory": 4096,
                            },
                            input=pipeline.input,
                            output=pipeline.output,
                            label='{0:s}:: Apply flag mask ms={1:s}'.format(step, msname))
 
+            if pipeline.enable_task(config, 'flag_manual'):
+                rules = config['flag_manual']['rules']
+                for irule, rule in enumerate(rules):
+                    # a manual flagging rule has a pattern to match the MS name, followed by key:value pairs
+                    rule_elements = rule.split()
+                    if len(rule_elements) < 2 or not all(':' in el for el in rule_elements[1:]):
+                        raise ValueError(f"invalid flag_manual rule '{rule}'")
+                    pattern = rule_elements[0]
+                    keywords = {tuple(elem.split(":", 1)) for elem in rule_elements[1:]}
+                    # end of parsing block. Replace this with file if you like
+                    if not fnmatch.fnmatch(msname, pattern):
+                        continue
+                    caracal.log.info(f"adding manual flagging rule for {pattern}")
+                    step = f'{wname}-manual-ms{msiter}-{irule}'
+                    args = {
+                                   "vis": msname,
+                                   "mode": 'manual',
+                                   "flagbackup": False,
+                                   "field": fields,
+                           }
+                    args.update(keywords)
+                    recipe.add('cab/casa_flagdata', step, args,
+                               input=pipeline.input,
+                               output=pipeline.output,
+                               label=f'{step}::Flag ms={msname} using {rule}')
+
             if pipeline.enable_task(config, 'flag_rfi'):
                 step = '{0:s}-rfi-ms{1:d}'.format(wname, msiter)
                 if config['flag_rfi']["flagger"] == "aoflagger":
@@ -397,12 +401,14 @@ def worker(pipeline, recipe, config):
                             for ss in stdr.readlines():
                                 for pp in 'xx,xy,yx,yy,stokes-i,stokes-q,stokes-u,stokes-v'.split(','):
                                     if '<on-{0:s}>1</on-{0:s}>'.format(pp) in ss: flag_corr.append(pp)
-                        if (('stokes-u' in flag_corr or 'stokes-v' in flag_corr) and ('XY' not in ms_corr or 'YX' not in ms_corr)) or\
-                             ('xy' in flag_corr and 'XY' not in ms_corr) or\
-                             ('yx' in flag_corr and 'YX' not in ms_corr) or\
-                             (('stokes-i' in flag_corr or 'stokes-q' in flag_corr) and ('XX' not in ms_corr or 'YY' not in ms_corr)) or\
-                             ('xx' in flag_corr and 'XX' not in ms_corr) or\
-                             ('yy' in flag_corr and 'YY' not in ms_corr):
+                        if ('stokes-u' in flag_corr and (('XY' not in ms_corr and 'RL' not in ms_corr) or ('YX' not in ms_corr and 'LR' not in ms_corr))) or\
+                             ('stokes-v' in flag_corr and (('XY' not in ms_corr and 'RR' not in ms_corr) or ('YX' not in ms_corr and 'LL' not in ms_corr))) or\
+                             ('stokes-i' in flag_corr and (('XX' not in ms_corr and 'RR' not in ms_corr) or ('YY' not in ms_corr and 'LL' not in ms_corr))) or\
+                             ('stokes-q' in flag_corr and (('XX' not in ms_corr and 'RL' not in ms_corr) or ('YY' not in ms_corr and 'LR' not in ms_corr))) or\
+                             ('xy' in flag_corr and ('XY' not in ms_corr and 'RL' not in ms_corr )) or\
+                             ('yx' in flag_corr and ('YX' not in ms_corr and 'LR' not in ms_corr)) or\
+                             ('xx' in flag_corr and ('XX' not in ms_corr and 'RR' not in ms_corr)) or\
+                             ('yy' in flag_corr and ('YY' not in ms_corr and 'LL' not in ms_corr)):
                             raise ValueError("The selected flagging strategy {0:s}/{1:s} will attempt to flag on {2:} but this is"\
                                              " not compatible with the {3:} correlations available in {4:s}. To proceed you can edit the flagging"\
                                              " strategy or, if you know what you are doing, disable aoflagger: ensure_valid.".format(
@@ -414,6 +420,9 @@ def worker(pipeline, recipe, config):
                                    "column": config['flag_rfi']['col'],
                                    "fields": ",".join(map(str, field_ids)),
                                    "strategy": config['flag_rfi']['aoflagger']['strategy'],
+                                   "indirect-read": True if config['flag_rfi']['aoflagger']['readmode'] == 'indirect' else False,
+                                   "memory-read": True if config['flag_rfi']['aoflagger']['readmode'] == 'memory' else False,
+                                   "auto-read-mode": True if config['flag_rfi']['aoflagger']['readmode'] == 'auto' else False,
                                },
                                input=pipeline.input,
                                output=pipeline.output,
@@ -468,10 +477,9 @@ def worker(pipeline, recipe, config):
             if pipeline.enable_task(config, 'inspect'):
                 step = '{0:s}-inspect-ms{1:d}'.format(wname,msiter)
                 if config['field'] == 'target':
-                    fieldName = utils.filter_name(target_ls[j])
                     field = '0'
                 else:
-                    field = ",".join(map(str, utils.get_field_id(msinfo, manfields.get_field(
+                    field = ",".join(map(str, utils.get_field_id(msdict, manfields.get_field(
                         pipeline, i, config['inspect']['field']).split(","))))
                 for f in field.split(','):
                     outlabel = '_{0:d}'.format(i) if len(field.split(',')) == 1 else '_{0:d}_{1:s}'.format(i,f)
@@ -499,24 +507,22 @@ def worker(pipeline, recipe, config):
             if pipeline.enable_task(config, 'summary'):
                 __label = config['label_in']
                 step = '{0:s}-summary-ms{1:d}'.format(wname,msiter)
-                recipe.add('cab/casa_flagdata', step,
-                           {
-                               "vis": msname,
-                               "mode": 'summary',
-                               "field": fields,
-                               "flagbackup": False,
+                recipe.add('cab/flagstats', step, {
+                           "msname"  : msname,
+                           "plot": True,
+                           "outfile": ('{0:s}-{1:s}-'
+                                       'flagging-summary-{2:d}.json').format(
+                                       prefix, wname, i),
+                           "htmlfile" : ('{0:s}-{1:s}-'
+                                         'flagging-summary-plots-{2:d}.html').format(
+                                         prefix, wname, i)
                            },
                            input=pipeline.input,
-                           output=pipeline.output,
+                           output=pipeline.diagnostic_plots,
                            label='{0:s}:: Flagging summary  ms={1:s}'.format(step, msname))
                 recipe.run()
                 # Empty job que after execution
                 recipe.jobs = []
-                summary_log = glob.glob("{0:s}/log-flag-{1:s}-*.txt".format(pipeline.logs, 
-                    step))[0]
-                json_summary = manflags.get_json_flag_summary(pipeline, summary_log,
-                                                              prefix, wname)
-                manflags.flag_summary_plots(pipeline, json_summary, prefix, wname, i)
 
 
             substep = 'save-{0:s}-ms{1:d}'.format(flags_after_worker, msiter)
